"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.encrypt = encrypt;
exports.decrypt = decrypt;
exports.unsafeDecrypt = unsafeDecrypt;

var _crypto = _interopRequireDefault(require("crypto"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Reference: http://lollyrock.com/articles/nodejs-encryption
const algorithm = 'aes-256-cbc';

function encrypt(plainText, secret) {
  if (!plainText) {
    throw new Error('Missing plain text');
  } else if (!secret) {
    throw new Error('Missing encrypt secret');
  }

  let key = Buffer.alloc(32);
  key = Buffer.concat([Buffer.from(secret)], key.length);

  const ivBytes = _crypto.default.randomBytes(16);

  const cipher = _crypto.default.createCipheriv(algorithm, key, ivBytes);

  return {
    ivText: ivBytes.toString('base64'),
    encryptedText: cipher.update(plainText, 'utf8', 'base64') + cipher.final('base64')
  };
}

function decrypt(encrypted, secret) {
  if (!encrypted || !encrypted.ivText || !encrypted.encryptedText) {
    throw new Error('Invalid encrypted valued');
  } else if (!secret) {
    throw new Error('Missing decrypt secret');
  }

  const iv = Buffer.from(encrypted.ivText, 'base64');
  let key = Buffer.alloc(32);
  key = Buffer.concat([Buffer.from(secret)], key.length);

  if (iv.length !== 16) {
    throw new Error('The encrypted value is not a valid format');
  }

  if (key.length !== 32) {
    throw new Error('The secret is not valid format');
  }

  const decipher = _crypto.default.createDecipheriv(algorithm, key, iv);

  return decipher.update(encrypted.encryptedText, 'base64', 'utf8') + decipher.final('utf8');
}
/**
 * Decrypts a value using the insecure createDecipher method.
 *
 * This method should not be used and only exists to give an upgrade path
 * of existing sqlectron-core users to the much better iv functions above.
 *
 * @deprecated 8.1.0
 * @param {string} text
 * @param {string} secret
 */


function unsafeDecrypt(text, secret) {
  if (!secret) {
    throw new Error('Missing crypto secret');
  }

  const decipher = _crypto.default.createDecipher('aes-256-ctr', secret);

  return decipher.update(text, 'hex', 'utf8') + decipher.final('utf8');
}