"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createConnection = createConnection;
exports.clearLimitSelect = clearLimitSelect;

var _tunnel = _interopRequireDefault(require("./tunnel"));

var _clients = _interopRequireDefault(require("./clients"));

var config = _interopRequireWildcard(require("../config"));

var _logger = _interopRequireDefault(require("../logger"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const logger = (0, _logger.default)('db');
const DEFAULT_LIMIT = 1000;
let limitSelect = null;

function createConnection(server, database) {
  /**
   * Database public API
   */
  return {
    connect: connect.bind(null, server, database),
    disconnect: disconnect.bind(null, server, database),
    version: version.bind(null, server, database),
    getVersion: getVersion.bind(null, server, database),
    listTables: listTables.bind(null, server, database),
    listViews: listViews.bind(null, server, database),
    listRoutines: listRoutines.bind(null, server, database),
    listTableColumns: listTableColumns.bind(null, server, database),
    listTableTriggers: listTableTriggers.bind(null, server, database),
    listTableIndexes: listTableIndexes.bind(null, server, database),
    listSchemas: listSchemas.bind(null, server, database),
    getTableReferences: getTableReferences.bind(null, server, database),
    getTableKeys: getTableKeys.bind(null, server, database),
    query: query.bind(null, server, database),
    executeQuery: executeQuery.bind(null, server, database),
    listDatabases: listDatabases.bind(null, server, database),
    getQuerySelectTop: getQuerySelectTop.bind(null, server, database),
    getTableCreateScript: getTableCreateScript.bind(null, server, database),
    getTableSelectScript: getTableSelectScript.bind(null, server, database),
    getTableInsertScript: getTableInsertScript.bind(null, server, database),
    getTableUpdateScript: getTableUpdateScript.bind(null, server, database),
    getTableDeleteScript: getTableDeleteScript.bind(null, server, database),
    getViewCreateScript: getViewCreateScript.bind(null, server, database),
    getRoutineCreateScript: getRoutineCreateScript.bind(null, server, database),
    truncateAllTables: truncateAllTables.bind(null, server, database)
  };
}

function clearLimitSelect() {
  limitSelect = null;
}

function connect(_x, _x2) {
  return _connect.apply(this, arguments);
}

function _connect() {
  _connect = _asyncToGenerator(function* (server, database) {
    /* eslint no-param-reassign: 0 */
    if (database.connecting) {
      throw new Error('There is already a connection in progress for this database. Aborting this new request.');
    }

    try {
      database.connecting = true; // terminate any previous lost connection for this DB

      if (database.connection) {
        database.connection.disconnect();
      } // reuse existing tunnel


      if (server.config.ssh && !server.sshTunnel) {
        logger().debug('creating ssh tunnel');
        server.sshTunnel = yield (0, _tunnel.default)(server.config);

        const _server$sshTunnel$add = server.sshTunnel.address(),
              address = _server$sshTunnel$add.address,
              port = _server$sshTunnel$add.port;

        logger().debug('ssh forwarding through local connection %s:%d', address, port);
        server.config.localHost = address;
        server.config.localPort = port;
      }

      const driver = _clients.default[server.config.client];

      const _yield$Promise$all = yield Promise.all([driver(server, database), handleSSHError(server.sshTunnel)]),
            _yield$Promise$all2 = _slicedToArray(_yield$Promise$all, 1),
            connection = _yield$Promise$all2[0];

      database.connection = connection;
    } catch (err) {
      logger().error('Connection error %j', err);
      disconnect(server, database);
      throw err;
    } finally {
      database.connecting = false;
    }
  });
  return _connect.apply(this, arguments);
}

function handleSSHError(sshTunnel) {
  return new Promise((resolve, reject) => {
    if (!sshTunnel) {
      return resolve();
    }

    sshTunnel.on('success', resolve);
    sshTunnel.on('error', error => {
      logger().error('ssh error %j', error);
      reject(error);
    });
  });
}

function disconnect(server, database) {
  database.connecting = false;

  if (database.connection) {
    database.connection.disconnect();
    database.connection = null;
  }

  if (server.db[database.database]) {
    delete server.db[database.database];
  }
}
/**
 * @deprecated
 */


function version(server, database) {
  checkIsConnected(server, database);
  return database.connection.version;
}

function getVersion(server, database) {
  checkIsConnected(server, database);
  return database.connection.getVersion();
}

function listSchemas(server, database, filter) {
  checkIsConnected(server, database);
  return database.connection.listSchemas(database.database, filter);
}

function listTables(server, database, filter) {
  checkIsConnected(server, database);
  return database.connection.listTables(database.database, filter);
}

function listViews(server, database, filter) {
  checkIsConnected(server, database);
  return database.connection.listViews(filter);
}

function listRoutines(server, database, filter) {
  checkIsConnected(server, database);
  return database.connection.listRoutines(filter);
}

function listTableColumns(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.listTableColumns(database.database, table, schema);
}

function listTableTriggers(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.listTableTriggers(table, schema);
}

function listTableIndexes(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.listTableIndexes(database.database, table, schema);
}

function getTableReferences(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.getTableReferences(table, schema);
}

function getTableKeys(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.getTableKeys(database.database, table, schema);
}

function query(server, database, queryText) {
  checkIsConnected(server, database);
  return database.connection.query(queryText);
}

function executeQuery(server, database, queryText) {
  checkIsConnected(server, database);
  return database.connection.executeQuery(queryText);
}

function listDatabases(server, database, filter) {
  checkIsConnected(server, database);
  return database.connection.listDatabases(filter);
}

function getQuerySelectTop(_x3, _x4, _x5, _x6, _x7) {
  return _getQuerySelectTop.apply(this, arguments);
}

function _getQuerySelectTop() {
  _getQuerySelectTop = _asyncToGenerator(function* (server, database, table, schema, limit) {
    checkIsConnected(server, database);
    let limitValue = limit;

    if (typeof limit === 'undefined') {
      yield loadConfigLimit();
      limitValue = typeof limitSelect !== 'undefined' && limitSelect !== null ? limitSelect : DEFAULT_LIMIT;
    }

    return database.connection.getQuerySelectTop(table, limitValue, schema);
  });
  return _getQuerySelectTop.apply(this, arguments);
}

function getTableCreateScript(server, database, table, schema) {
  checkIsConnected(server, database);
  return database.connection.getTableCreateScript(table, schema);
}

function getTableSelectScript(_x8, _x9, _x10, _x11) {
  return _getTableSelectScript.apply(this, arguments);
}

function _getTableSelectScript() {
  _getTableSelectScript = _asyncToGenerator(function* (server, database, table, schema) {
    const columnNames = yield getTableColumnNames(server, database, table, schema);
    const schemaSelection = resolveSchema(database, schema);
    return [`SELECT ${wrap(database, columnNames).join(', ')}`, `FROM ${schemaSelection}${wrap(database, table)};`].join(' ');
  });
  return _getTableSelectScript.apply(this, arguments);
}

function getTableInsertScript(_x12, _x13, _x14, _x15) {
  return _getTableInsertScript.apply(this, arguments);
}

function _getTableInsertScript() {
  _getTableInsertScript = _asyncToGenerator(function* (server, database, table, schema) {
    const columnNames = yield getTableColumnNames(server, database, table, schema);
    const schemaSelection = resolveSchema(database, schema);
    return [`INSERT INTO ${schemaSelection}${wrap(database, table)}`, `(${wrap(database, columnNames).join(', ')})\n`, `VALUES (${columnNames.fill('?').join(', ')});`].join(' ');
  });
  return _getTableInsertScript.apply(this, arguments);
}

function getTableUpdateScript(_x16, _x17, _x18, _x19) {
  return _getTableUpdateScript.apply(this, arguments);
}

function _getTableUpdateScript() {
  _getTableUpdateScript = _asyncToGenerator(function* (server, database, table, schema) {
    const columnNames = yield getTableColumnNames(server, database, table, schema);
    const setColumnForm = wrap(database, columnNames).map(col => `${col}=?`).join(', ');
    const schemaSelection = resolveSchema(database, schema);
    return [`UPDATE ${schemaSelection}${wrap(database, table)}\n`, `SET ${setColumnForm}\n`, 'WHERE <condition>;'].join(' ');
  });
  return _getTableUpdateScript.apply(this, arguments);
}

function getTableDeleteScript(server, database, table, schema) {
  const schemaSelection = resolveSchema(database, schema);
  return [`DELETE FROM ${schemaSelection}${wrap(database, table)}`, 'WHERE <condition>;'].join(' ');
}

function getViewCreateScript(server, database, view
/* , schema */
) {
  checkIsConnected(server, database);
  return database.connection.getViewCreateScript(view);
}

function getRoutineCreateScript(server, database, routine, type, schema) {
  checkIsConnected(server, database);
  return database.connection.getRoutineCreateScript(routine, type, schema);
}

function truncateAllTables(server, database, schema) {
  return database.connection.truncateAllTables(database.database, schema);
}

function getTableColumnNames(_x20, _x21, _x22, _x23) {
  return _getTableColumnNames.apply(this, arguments);
}

function _getTableColumnNames() {
  _getTableColumnNames = _asyncToGenerator(function* (server, database, table, schema) {
    checkIsConnected(server, database);
    const columns = yield database.connection.listTableColumns(database.database, table, schema);
    return columns.map(column => column.columnName);
  });
  return _getTableColumnNames.apply(this, arguments);
}

function resolveSchema(database, schema) {
  return schema ? `${wrap(database, schema)}.` : '';
}

function wrap(database, identifier) {
  if (!Array.isArray(identifier)) {
    return database.connection.wrapIdentifier(identifier);
  }

  return identifier.map(item => database.connection.wrapIdentifier(item));
}

function loadConfigLimit() {
  return _loadConfigLimit.apply(this, arguments);
}

function _loadConfigLimit() {
  _loadConfigLimit = _asyncToGenerator(function* () {
    if (typeof limitSelect === 'undefined' || limitSelect === null) {
      const _yield$config$get = yield config.get(),
            limitQueryDefaultSelectTop = _yield$config$get.limitQueryDefaultSelectTop;

      limitSelect = limitQueryDefaultSelectTop;
    }

    return limitSelect;
  });
  return _loadConfigLimit.apply(this, arguments);
}

function checkIsConnected(server, database) {
  if (database.connecting || !database.connection) {
    throw new Error('There is no connection available.');
  }
}