"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
exports.disconnect = disconnect;
exports.listTables = listTables;
exports.listViews = listViews;
exports.listRoutines = listRoutines;
exports.listTableColumns = listTableColumns;
exports.listTableTriggers = listTableTriggers;
exports.listTableIndexes = listTableIndexes;
exports.listSchemas = listSchemas;
exports.getTableReferences = getTableReferences;
exports.getTableKeys = getTableKeys;
exports.executeQuery = executeQuery;
exports.listDatabases = listDatabases;
exports.getQuerySelectTop = getQuerySelectTop;
exports.getTableCreateScript = getTableCreateScript;
exports.getViewCreateScript = getViewCreateScript;
exports.getRoutineCreateScript = getRoutineCreateScript;
exports.wrapIdentifier = wrapIdentifier;
exports.truncateAllTables = void 0;

var cassandra = _interopRequireWildcard(require("cassandra-driver"));

var _sqlQueryIdentifier = require("sql-query-identifier");

var _logger = _interopRequireDefault(require("../../logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const logger = (0, _logger.default)('db:clients:cassandra');
/**
 * To keep compatibility with the other clients we treat keyspaces as database.
 */

function _default(server, database) {
  return new Promise( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator(function* (resolve, reject) {
      const dbConfig = configDatabase(server, database);
      logger().debug('creating database client %j', dbConfig);
      const client = new cassandra.Client(dbConfig);
      logger().debug('connecting');
      client.connect( /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (err) {
          if (err) {
            client.shutdown();
            return reject(err);
          }

          client.version = client.getState().getConnectedHosts()[0].getCassandraVersion();
          client.versionData = {
            name: 'Cassandra',
            version: client.getState().getConnectedHosts()[0].cassandraVersion,
            string: `Cassandra ${client.getState().getConnectedHosts()[0].cassandraVersion}`
          };
          logger().debug('connected');
          resolve({
            wrapIdentifier,
            version: client.getState().getConnectedHosts()[0].cassandraVersion,
            getVersion: () => client.versionData,
            disconnect: () => disconnect(client),
            listTables: db => listTables(client, db),
            listViews: () => listViews(client),
            listRoutines: () => listRoutines(client),
            listTableColumns: (db, table) => listTableColumns(client, db, table),
            listTableTriggers: table => listTableTriggers(client, table),
            listTableIndexes: (db, table) => listTableIndexes(client, table),
            listSchemas: () => listSchemas(client),
            getTableReferences: table => getTableReferences(client, table),
            getTableKeys: (db, table) => getTableKeys(client, db, table),
            query: queryText => executeQuery(client, queryText),
            executeQuery: queryText => executeQuery(client, queryText),
            listDatabases: () => listDatabases(client),
            getQuerySelectTop: (table, limit) => getQuerySelectTop(client, table, limit),
            getTableCreateScript: table => getTableCreateScript(client, table),
            getViewCreateScript: view => getViewCreateScript(client, view),
            getRoutineCreateScript: routine => getRoutineCreateScript(client, routine),
            truncateAllTables: db => truncateAllTables(client, db)
          });
        });

        return function (_x3) {
          return _ref2.apply(this, arguments);
        };
      }());
    });

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }());
}

function disconnect(client) {
  client.shutdown();
}

function listTables(client, database) {
  return new Promise((resolve, reject) => {
    let sql;

    if (client.version[0] === 2) {
      sql = `
        SELECT columnfamily_name as name
        FROM system.schema_columnfamilies
        WHERE keyspace_name = ?
      `;
    } else {
      sql = `
        SELECT table_name as name
        FROM system_schema.tables
        WHERE keyspace_name = ?
      `;
    }

    const params = [database];
    client.execute(sql, params, (err, data) => {
      if (err) return reject(err);
      resolve(data.rows.map(row => ({
        name: row.name
      })));
    });
  });
}

function listViews() {
  return Promise.resolve([]);
}

function listRoutines() {
  return Promise.resolve([]);
}

function listTableColumns(client, database, table) {
  const cassandra2 = client.version[0] === 2;
  return new Promise((resolve, reject) => {
    let sql;

    if (cassandra2) {
      sql = `
        SELECT type as position, column_name, validator as type
        FROM system.schema_columns
        WHERE keyspace_name = ?
          AND columnfamily_name = ?
      `;
    } else {
      sql = `
        SELECT position, column_name, type
        FROM system_schema.columns
        WHERE keyspace_name = ?
          AND table_name = ?
      `;
    }

    const params = [database, table];
    client.execute(sql, params, (err, data) => {
      if (err) return reject(err);
      resolve(data.rows // force pks be placed at the results beginning
      .sort((a, b) => {
        if (cassandra2) {
          return +(a.position > b.position) || -(a.position < b.position);
        }

        return b.position - a.position;
      }).map(row => {
        const rowType = cassandra2 ? mapLegacyDataTypes(row.type) : row.type;
        return {
          columnName: row.column_name,
          dataType: rowType
        };
      }));
    });
  });
}
/**
 * The system schema of Casandra 2.x does not have data type, but only validator
 * classes. To make the behavior consistent with v3.x, we try to deduce the
 * correponding CQL data type using the validator name.
 *
 * @param {string} validator
 * @returns {string}
 */


function mapLegacyDataTypes(validator) {
  const type = validator.split('.').pop();

  switch (type) {
    case 'Int32Type':
    case 'LongType':
      return 'int';

    case 'UTF8Type':
      return 'text';

    case 'TimestampType':
    case 'DateType':
      return 'timestamp';

    case 'DoubleType':
      return 'double';

    case 'FloatType':
      return 'float';

    case 'UUIDType':
      return 'uuid';

    case 'CounterColumnType':
      return 'counter';

    default:
      logger().debug('validator %s is not yet mapped!', validator);
      return type;
  }
}

function listTableTriggers() {
  return Promise.resolve([]);
}

function listTableIndexes() {
  return Promise.resolve([]);
}

function listSchemas() {
  return Promise.resolve([]);
}

function getTableReferences() {
  return Promise.resolve([]);
}

function getTableKeys(client, database, table) {
  return client.metadata.getTable(database, table).then(tableInfo => tableInfo.partitionKeys.map(key => ({
    constraintName: null,
    columnName: key.name,
    referencedTable: null,
    keyType: 'PRIMARY KEY'
  })));
}

function query(conn, queryText) {
  // eslint-disable-line no-unused-vars
  throw new Error('"query" function is not implementd by cassandra client.');
}

function executeQuery(client, queryText) {
  const commands = identifyCommands(queryText).map(item => item.type);
  return new Promise((resolve, reject) => {
    client.execute(queryText, (err, data) => {
      if (err) return reject(err);
      resolve([parseRowQueryResult(data, commands[0])]);
    });
  });
}

function listDatabases(client) {
  return new Promise(resolve => {
    resolve(Object.keys(client.metadata.keyspaces));
  });
}

function getQuerySelectTop(client, table, limit) {
  return `SELECT * FROM ${wrapIdentifier(table)} LIMIT ${limit}`;
}

function getTableCreateScript() {
  return Promise.resolve([]);
}

function getViewCreateScript() {
  return Promise.resolve([]);
}

function getRoutineCreateScript() {
  return Promise.resolve([]);
}

function wrapIdentifier(value) {
  if (value === '*') return value;
  const matched = value.match(/(.*?)(\[[0-9]\])/); // eslint-disable-line no-useless-escape

  if (matched) return wrapIdentifier(matched[1]) + matched[2];
  return `"${value.replace(/"/g, '""')}"`;
}

const truncateAllTables = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (connection, database) {
    const result = yield listTables(connection, database);
    const tables = result.map(table => table.name);
    const promises = tables.map(t => {
      const truncateSQL = `
      TRUNCATE TABLE ${wrapIdentifier(database)}.${wrapIdentifier(t)};
    `;
      return executeQuery(connection, truncateSQL);
    });
    yield Promise.all(promises);
  });

  return function truncateAllTables(_x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}();

exports.truncateAllTables = truncateAllTables;

function configDatabase(server, database) {
  const config = {
    contactPoints: [server.config.host],
    protocolOptions: {
      port: server.config.port
    },
    keyspace: database.database
  };

  if (server.sshTunnel) {
    config.contactPoints = [server.config.localHost];
    config.protocolOptions.port = server.config.localPort;
  }

  if (server.config.ssl) {} // TODO: sslOptions
  // client authentication


  if (server.config.user && server.config.password) {
    const user = server.config.user;
    const password = server.config.password;
    const authProviderInfo = new cassandra.auth.PlainTextAuthProvider(user, password);
    config.authProvider = authProviderInfo;
  }

  return config;
}

function parseRowQueryResult(data, command) {
  // Fallback in case the identifier could not reconize the command
  const isSelect = command ? command === 'SELECT' : Array.isArray(data.rows);
  return {
    command: command || isSelect && 'SELECT',
    rows: data.rows || [],
    fields: data.columns || [],
    rowCount: isSelect ? data.rowLength || 0 : undefined,
    affectedRows: !isSelect && !isNaN(data.rowLength) ? data.rowLength : undefined
  };
}

function identifyCommands(queryText) {
  try {
    return (0, _sqlQueryIdentifier.identify)(queryText);
  } catch (err) {
    return [];
  }
}