"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
exports.disconnect = disconnect;
exports.listTables = listTables;
exports.listViews = listViews;
exports.listRoutines = listRoutines;
exports.listTableColumns = listTableColumns;
exports.listTableTriggers = listTableTriggers;
exports.listTableIndexes = listTableIndexes;
exports.listSchemas = listSchemas;
exports.getTableReferences = getTableReferences;
exports.getTableKeys = getTableKeys;
exports.query = query;
exports.executeQuery = executeQuery;
exports.listDatabases = listDatabases;
exports.getQuerySelectTop = getQuerySelectTop;
exports.getTableCreateScript = getTableCreateScript;
exports.getViewCreateScript = getViewCreateScript;
exports.getRoutineCreateScript = getRoutineCreateScript;
exports.wrapIdentifier = wrapIdentifier;
exports.truncateAllTables = truncateAllTables;
exports.filterDatabase = filterDatabase;

var _mysql = _interopRequireDefault(require("mysql2"));

var _sqlQueryIdentifier = require("sql-query-identifier");

var _logger = _interopRequireDefault(require("../../logger"));

var _utils = require("../../utils");

var _errors = _interopRequireDefault(require("../../errors"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const logger = (0, _logger.default)('db:clients:mysql');
const mysqlErrors = {
  EMPTY_QUERY: 'ER_EMPTY_QUERY',
  CONNECTION_LOST: 'PROTOCOL_CONNECTION_LOST'
};

function _default(_x, _x2) {
  return _ref.apply(this, arguments);
}

function _ref() {
  _ref = _asyncToGenerator(function* (server, database) {
    const dbConfig = configDatabase(server, database);
    logger().debug('create driver client for mysql with config %j', dbConfig);
    const conn = {
      pool: _mysql.default.createPool(dbConfig)
    };
    const versionInfo = (yield driverExecuteQuery(conn, {
      query: "SHOW VARIABLES WHERE variable_name='version' OR variable_name='version_comment';"
    })).data;
    let version;
    let versionComment;

    for (let i = 0; i < versionInfo.length; i++) {
      const item = versionInfo[i];

      if (item.Variable_name === 'version') {
        version = item.Value;
      } else if (item.Variable_name === 'version_comment') {
        versionComment = item.Value;
      }
    }

    conn.versionData = {
      name: versionComment,
      version: version.split('-')[0],
      string: `${versionComment} ${version}`
    }; // normalize the name as depending on where the server is installed from, it
    // could be just "MySQL" or "MariaDB", or it could be a longer string like
    // "mariadb.org binary distribution"

    const lowerComment = versionComment.toLowerCase();

    if (lowerComment.includes('mysql')) {
      conn.versionData.name = 'MySQL';
    } else if (lowerComment.includes('mariadb')) {
      conn.versionData.name = 'MariaDB';
    } else if (lowerComment.includes('percona')) {
      conn.versionData.name = 'Percona';
    }

    return {
      wrapIdentifier,
      version: conn.versionData.version,
      getVersion: () => conn.versionData,
      disconnect: () => disconnect(conn),
      listTables: () => listTables(conn),
      listViews: () => listViews(conn),
      listRoutines: () => listRoutines(conn),
      listTableColumns: (db, table) => listTableColumns(conn, db, table),
      listTableTriggers: table => listTableTriggers(conn, table),
      listTableIndexes: (db, table) => listTableIndexes(conn, db, table),
      listSchemas: () => listSchemas(conn),
      getTableReferences: table => getTableReferences(conn, table),
      getTableKeys: (db, table) => getTableKeys(conn, db, table),
      query: queryText => query(conn, queryText),
      executeQuery: queryText => executeQuery(conn, queryText),
      listDatabases: filter => listDatabases(conn, filter),
      getQuerySelectTop: (table, limit) => getQuerySelectTop(conn, table, limit),
      getTableCreateScript: table => getTableCreateScript(conn, table),
      getViewCreateScript: view => getViewCreateScript(conn, view),
      getRoutineCreateScript: (routine, type) => getRoutineCreateScript(conn, routine, type),
      truncateAllTables: () => truncateAllTables(conn)
    };
  });
  return _ref.apply(this, arguments);
}

function disconnect(conn) {
  conn.pool.end();
}

function listTables(_x3) {
  return _listTables.apply(this, arguments);
}

function _listTables() {
  _listTables = _asyncToGenerator(function* (conn) {
    const sql = `
    SELECT table_name as name
    FROM information_schema.tables
    WHERE table_schema = database()
    AND table_type NOT LIKE '%VIEW%'
    ORDER BY table_name
  `;

    const _yield$driverExecuteQ2 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ2.data;

    return data;
  });
  return _listTables.apply(this, arguments);
}

function listViews(_x4) {
  return _listViews.apply(this, arguments);
}

function _listViews() {
  _listViews = _asyncToGenerator(function* (conn) {
    const sql = `
    SELECT table_name as name
    FROM information_schema.views
    WHERE table_schema = database()
    ORDER BY table_name
  `;

    const _yield$driverExecuteQ3 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ3.data;

    return data;
  });
  return _listViews.apply(this, arguments);
}

function listRoutines(_x5) {
  return _listRoutines.apply(this, arguments);
}

function _listRoutines() {
  _listRoutines = _asyncToGenerator(function* (conn) {
    const sql = `
    SELECT routine_name as 'routine_name', routine_type as 'routine_type'
    FROM information_schema.routines
    WHERE routine_schema = database()
    ORDER BY routine_name
  `;

    const _yield$driverExecuteQ4 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ4.data;

    return data.map(row => ({
      routineName: row.routine_name,
      routineType: row.routine_type
    }));
  });
  return _listRoutines.apply(this, arguments);
}

function listTableColumns(_x6, _x7, _x8) {
  return _listTableColumns.apply(this, arguments);
}

function _listTableColumns() {
  _listTableColumns = _asyncToGenerator(function* (conn, database, table) {
    const sql = `
    SELECT column_name AS 'column_name', data_type AS 'data_type'
    FROM information_schema.columns
    WHERE table_schema = database()
    AND table_name = ?
    ORDER BY ordinal_position
  `;
    const params = [table];

    const _yield$driverExecuteQ5 = yield driverExecuteQuery(conn, {
      query: sql,
      params
    }),
          data = _yield$driverExecuteQ5.data;

    return data.map(row => ({
      columnName: row.column_name,
      dataType: row.data_type
    }));
  });
  return _listTableColumns.apply(this, arguments);
}

function listTableTriggers(_x9, _x10) {
  return _listTableTriggers.apply(this, arguments);
}

function _listTableTriggers() {
  _listTableTriggers = _asyncToGenerator(function* (conn, table) {
    const sql = `
    SELECT trigger_name as 'trigger_name'
    FROM information_schema.triggers
    WHERE event_object_schema = database()
    AND event_object_table = ?
  `;
    const params = [table];

    const _yield$driverExecuteQ6 = yield driverExecuteQuery(conn, {
      query: sql,
      params
    }),
          data = _yield$driverExecuteQ6.data;

    return data.map(row => row.trigger_name);
  });
  return _listTableTriggers.apply(this, arguments);
}

function listTableIndexes(_x11, _x12, _x13) {
  return _listTableIndexes.apply(this, arguments);
}

function _listTableIndexes() {
  _listTableIndexes = _asyncToGenerator(function* (conn, database, table) {
    const sql = 'SHOW INDEX FROM ?? FROM ??';
    const params = [table, database];

    const _yield$driverExecuteQ7 = yield driverExecuteQuery(conn, {
      query: sql,
      params
    }),
          data = _yield$driverExecuteQ7.data;

    return data.map(row => row.Key_name);
  });
  return _listTableIndexes.apply(this, arguments);
}

function listSchemas() {
  return Promise.resolve([]);
}

function getTableReferences(_x14, _x15) {
  return _getTableReferences.apply(this, arguments);
}

function _getTableReferences() {
  _getTableReferences = _asyncToGenerator(function* (conn, table) {
    const sql = `
    SELECT referenced_table_name as 'referenced_table_name'
    FROM information_schema.key_column_usage
    WHERE referenced_table_name IS NOT NULL
    AND table_schema = database()
    AND table_name = ?
  `;
    const params = [table];

    const _yield$driverExecuteQ8 = yield driverExecuteQuery(conn, {
      query: sql,
      params
    }),
          data = _yield$driverExecuteQ8.data;

    return data.map(row => row.referenced_table_name);
  });
  return _getTableReferences.apply(this, arguments);
}

function getTableKeys(_x16, _x17, _x18) {
  return _getTableKeys.apply(this, arguments);
}

function _getTableKeys() {
  _getTableKeys = _asyncToGenerator(function* (conn, database, table) {
    const sql = `
    SELECT
      constraint_name as 'constraint_name',
      column_name as 'column_name',
      referenced_table_name as 'referenced_table_name',
      CASE WHEN (referenced_table_name IS NOT NULL) THEN 'FOREIGN'
      ELSE constraint_name
      END as key_type
    FROM information_schema.key_column_usage
    WHERE table_schema = database()
    AND table_name = ?
    AND ((referenced_table_name IS NOT NULL) OR constraint_name LIKE '%PRIMARY%')
  `;
    const params = [table];

    const _yield$driverExecuteQ9 = yield driverExecuteQuery(conn, {
      query: sql,
      params
    }),
          data = _yield$driverExecuteQ9.data;

    return data.map(row => ({
      constraintName: `${row.constraint_name} KEY`,
      columnName: row.column_name,
      referencedTable: row.referenced_table_name,
      keyType: `${row.key_type} KEY`
    }));
  });
  return _getTableKeys.apply(this, arguments);
}

function query(conn, queryText) {
  let pid = null;
  let canceling = false;
  const cancelable = (0, _utils.createCancelablePromise)(_objectSpread(_objectSpread({}, _errors.default.CANCELED_BY_USER), {}, {
    sqlectronError: 'CANCELED_BY_USER'
  }));
  return {
    execute() {
      return runWithConnection(conn, /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (connection) {
          const connClient = {
            connection
          };

          const _yield$driverExecuteQ = yield driverExecuteQuery(connClient, {
            query: 'SELECT connection_id() AS pid'
          }),
                dataPid = _yield$driverExecuteQ.data;

          pid = dataPid[0].pid;

          try {
            const data = yield Promise.race([cancelable.wait(), executeQuery(connClient, queryText)]);
            pid = null;
            return data;
          } catch (err) {
            if (canceling && err.code === mysqlErrors.CONNECTION_LOST) {
              canceling = false;
              err.sqlectronError = 'CANCELED_BY_USER';
            }

            throw err;
          } finally {
            cancelable.discard();
          }
        });

        return function (_x19) {
          return _ref2.apply(this, arguments);
        };
      }());
    },

    cancel() {
      return _asyncToGenerator(function* () {
        if (!pid) {
          throw new Error('Query not ready to be canceled');
        }

        canceling = true;

        try {
          yield driverExecuteQuery(conn, {
            query: `kill ${pid};`
          });
          cancelable.cancel();
        } catch (err) {
          canceling = false;
          throw err;
        }
      })();
    }

  };
}

function executeQuery(_x20, _x21) {
  return _executeQuery.apply(this, arguments);
}

function _executeQuery() {
  _executeQuery = _asyncToGenerator(function* (conn, queryText) {
    const _yield$driverExecuteQ10 = yield driverExecuteQuery(conn, {
      query: queryText
    }),
          fields = _yield$driverExecuteQ10.fields,
          data = _yield$driverExecuteQ10.data;

    if (!data) {
      return [];
    }

    const commands = identifyCommands(queryText).map(item => item.type);

    if (!isMultipleQuery(fields)) {
      return [parseRowQueryResult(data, fields, commands[0])];
    }

    return data.map((_, idx) => parseRowQueryResult(data[idx], fields[idx], commands[idx]));
  });
  return _executeQuery.apply(this, arguments);
}

function listDatabases(_x22, _x23) {
  return _listDatabases.apply(this, arguments);
}

function _listDatabases() {
  _listDatabases = _asyncToGenerator(function* (conn, filter) {
    const sql = 'show databases';

    const _yield$driverExecuteQ11 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ11.data;

    return data.filter(item => filterDatabase(item, filter, 'Database')).map(row => row.Database);
  });
  return _listDatabases.apply(this, arguments);
}

function getQuerySelectTop(conn, table, limit) {
  return `SELECT * FROM ${wrapIdentifier(table)} LIMIT ${limit}`;
}

function getTableCreateScript(_x24, _x25) {
  return _getTableCreateScript.apply(this, arguments);
}

function _getTableCreateScript() {
  _getTableCreateScript = _asyncToGenerator(function* (conn, table) {
    const sql = `SHOW CREATE TABLE ${table}`;

    const _yield$driverExecuteQ12 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ12.data;

    return data.map(row => row['Create Table']);
  });
  return _getTableCreateScript.apply(this, arguments);
}

function getViewCreateScript(_x26, _x27) {
  return _getViewCreateScript.apply(this, arguments);
}

function _getViewCreateScript() {
  _getViewCreateScript = _asyncToGenerator(function* (conn, view) {
    const sql = `SHOW CREATE VIEW ${view}`;

    const _yield$driverExecuteQ13 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ13.data;

    return data.map(row => row['Create View']);
  });
  return _getViewCreateScript.apply(this, arguments);
}

function getRoutineCreateScript(_x28, _x29, _x30) {
  return _getRoutineCreateScript.apply(this, arguments);
}

function _getRoutineCreateScript() {
  _getRoutineCreateScript = _asyncToGenerator(function* (conn, routine, type) {
    const sql = `SHOW CREATE ${type.toUpperCase()} ${routine}`;

    const _yield$driverExecuteQ14 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ14.data;

    return data.map(row => row[`Create ${type}`]);
  });
  return _getRoutineCreateScript.apply(this, arguments);
}

function wrapIdentifier(value) {
  return value !== '*' ? `\`${value.replace(/`/g, '``')}\`` : '*';
}

function getSchema(_x31) {
  return _getSchema.apply(this, arguments);
}

function _getSchema() {
  _getSchema = _asyncToGenerator(function* (conn) {
    const sql = 'SELECT database() AS \'schema\'';

    const _yield$driverExecuteQ15 = yield driverExecuteQuery(conn, {
      query: sql
    }),
          data = _yield$driverExecuteQ15.data;

    return data[0].schema;
  });
  return _getSchema.apply(this, arguments);
}

function truncateAllTables(_x32) {
  return _truncateAllTables.apply(this, arguments);
}

function _truncateAllTables() {
  _truncateAllTables = _asyncToGenerator(function* (conn) {
    yield runWithConnection(conn, /*#__PURE__*/function () {
      var _ref3 = _asyncToGenerator(function* (connection) {
        const connClient = {
          connection
        };
        const schema = yield getSchema(connClient);
        const sql = `
      SELECT table_name as 'table_name'
      FROM information_schema.tables
      WHERE table_schema = '${schema}'
      AND table_type NOT LIKE '%VIEW%'
    `;

        const _yield$driverExecuteQ16 = yield driverExecuteQuery(connClient, {
          query: sql
        }),
              data = _yield$driverExecuteQ16.data;

        const truncateAll = data.map(row => `
      SET FOREIGN_KEY_CHECKS = 0;
      TRUNCATE TABLE ${wrapIdentifier(schema)}.${wrapIdentifier(row.table_name)};
      SET FOREIGN_KEY_CHECKS = 1;
    `).join('');
        yield driverExecuteQuery(connClient, {
          query: truncateAll
        });
      });

      return function (_x35) {
        return _ref3.apply(this, arguments);
      };
    }());
  });
  return _truncateAllTables.apply(this, arguments);
}

function configDatabase(server, database) {
  const config = {
    host: server.config.host,
    port: server.config.port,
    user: server.config.user,
    password: server.config.password,
    database: database.database,
    multipleStatements: true,
    dateStrings: true,
    supportBigNumbers: true,
    bigNumberStrings: true
  };

  if (server.sshTunnel) {
    config.host = server.config.localHost;
    config.port = server.config.localPort;
  }

  if (server.config.ssl) {
    config.ssl = {
      // It is not the best recommend way to use SSL with node-mysql
      // https://github.com/felixge/node-mysql#ssl-options
      // But this way we have compatibility with all clients.
      rejectUnauthorized: false
    };
  }

  return config;
}

function getRealError(conn, err) {
  /* eslint no-underscore-dangle:0 */
  if (conn && conn._protocol && conn._protocol._fatalError) {
    return conn._protocol._fatalError;
  }

  return err;
}

function parseRowQueryResult(data, fields, command) {
  // Fallback in case the identifier could not reconize the command
  const isSelect = Array.isArray(data);
  return {
    command: command || isSelect && 'SELECT',
    rows: isSelect ? data : [],
    fields: fields || [],
    rowCount: isSelect ? (data || []).length : undefined,
    affectedRows: !isSelect ? data.affectedRows : undefined
  };
}

function isMultipleQuery(fields) {
  if (!fields) {
    return false;
  }

  if (!fields.length) {
    return false;
  }

  return Array.isArray(fields[0]) || fields[0] === undefined;
}

function identifyCommands(queryText) {
  try {
    return (0, _sqlQueryIdentifier.identify)(queryText);
  } catch (err) {
    return [];
  }
}

function driverExecuteQuery(conn, queryArgs) {
  const runQuery = connection => new Promise((resolve, reject) => {
    connection.query(queryArgs.query, queryArgs.params, (err, data, fields) => {
      if (err && err.code === mysqlErrors.EMPTY_QUERY) return resolve({});
      if (err) return reject(getRealError(connection, err));
      resolve({
        data,
        fields
      });
    });
  });

  return conn.connection ? runQuery(conn.connection) : runWithConnection(conn, runQuery);
}

function runWithConnection(_x33, _x34) {
  return _runWithConnection.apply(this, arguments);
}

function _runWithConnection() {
  _runWithConnection = _asyncToGenerator(function* ({
    pool
  }, run) {
    let rejected = false;
    return new Promise((resolve, reject) => {
      const rejectErr = err => {
        if (!rejected) {
          rejected = true;
          reject(err);
        }
      };

      pool.getConnection( /*#__PURE__*/function () {
        var _ref4 = _asyncToGenerator(function* (errPool, connection) {
          if (errPool) {
            rejectErr(errPool);
            return;
          }

          connection.on('error', error => {
            // it will be handled later in the next query execution
            logger().error('Connection fatal error %j', error);
          });

          try {
            resolve(yield run(connection));
          } catch (err) {
            rejectErr(err);
          } finally {
            connection.release();
          }
        });

        return function (_x36, _x37) {
          return _ref4.apply(this, arguments);
        };
      }());
    });
  });
  return _runWithConnection.apply(this, arguments);
}

function filterDatabase(item, {
  database
} = {}, databaseField) {
  if (!database) {
    return true;
  }

  const value = item[databaseField];

  if (typeof database === 'string') {
    return database === value;
  }

  const only = database.only,
        ignore = database.ignore;

  if (only && only.length && !~only.indexOf(value)) {
    return false;
  }

  if (ignore && ignore.length && ~ignore.indexOf(value)) {
    return false;
  }

  return true;
}