"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
exports.disconnect = disconnect;
exports.listTables = listTables;
exports.listViews = listViews;
exports.listRoutines = listRoutines;
exports.listTableColumns = listTableColumns;
exports.listTableTriggers = listTableTriggers;
exports.listTableIndexes = listTableIndexes;
exports.listSchemas = listSchemas;
exports.getTableReferences = getTableReferences;
exports.getTableKeys = getTableKeys;
exports.query = query;
exports.executeQuery = executeQuery;
exports.listDatabases = listDatabases;
exports.getQuerySelectTop = getQuerySelectTop;
exports.getTableCreateScript = getTableCreateScript;
exports.getViewCreateScript = getViewCreateScript;
exports.getRoutineCreateScript = getRoutineCreateScript;
exports.wrapIdentifier = wrapIdentifier;
exports.truncateAllTables = truncateAllTables;

var _pg = _interopRequireDefault(require("pg"));

var _sqlQueryIdentifier = require("sql-query-identifier");

var _utils = require("./utils");

var _logger = _interopRequireDefault(require("../../logger"));

var _utils2 = require("../../utils");

var _errors = _interopRequireDefault(require("../../errors"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const logger = (0, _logger.default)('db:clients:postgresql');
const pgErrors = {
  CANCELED: '57014'
};
/**
 * Do not convert DATE types to JS date.
 * It gnores of applying a wrong timezone to the date.
 * TODO: do not convert as well these same types with array (types 1115, 1182, 1185)
 */

_pg.default.types.setTypeParser(1082, 'text', val => val); // date


_pg.default.types.setTypeParser(1114, 'text', val => val); // timestamp without timezone


_pg.default.types.setTypeParser(1184, 'text', val => val); // timestamp


function _default(_x, _x2) {
  return _ref.apply(this, arguments);
}

function _ref() {
  _ref = _asyncToGenerator(function* (server, database) {
    const dbConfig = configDatabase(server, database);
    logger().debug('create driver client for postgres with config %j', dbConfig);
    const conn = {
      pool: new _pg.default.Pool(dbConfig)
    };
    logger().debug('connected');
    const defaultSchema = yield getSchema(conn);
    const version = (yield driverExecuteQuery(conn, {
      query: 'select version()'
    })).rows[0].version;
    const splitVersion = version.split(' ');
    conn.versionData = {
      name: splitVersion[0],
      version: splitVersion[1],
      string: version
    };
    return {
      /* eslint max-len:0 */
      wrapIdentifier,
      version,
      getVersion: () => conn.versionData,
      disconnect: () => disconnect(conn),
      listTables: (db, filter) => listTables(conn, filter),
      listViews: filter => listViews(conn, filter),
      listRoutines: filter => listRoutines(conn, filter),
      listTableColumns: (db, table, schema = defaultSchema) => listTableColumns(conn, db, table, schema),
      listTableTriggers: (table, schema = defaultSchema) => listTableTriggers(conn, table, schema),
      listTableIndexes: (db, table, schema = defaultSchema) => listTableIndexes(conn, table, schema),
      listSchemas: (db, filter) => listSchemas(conn, filter),
      getTableReferences: (table, schema = defaultSchema) => getTableReferences(conn, table, schema),
      getTableKeys: (db, table, schema = defaultSchema) => getTableKeys(conn, db, table, schema),
      query: (queryText, schema = defaultSchema) => query(conn, queryText, schema),
      executeQuery: (queryText, schema = defaultSchema) => executeQuery(conn, queryText, schema),
      listDatabases: filter => listDatabases(conn, filter),
      getQuerySelectTop: (table, limit, schema = defaultSchema) => getQuerySelectTop(conn, table, limit, schema),
      getTableCreateScript: (table, schema = defaultSchema) => getTableCreateScript(conn, table, schema),
      getViewCreateScript: (view, schema = defaultSchema) => getViewCreateScript(conn, view, schema),
      getRoutineCreateScript: (routine, type, schema = defaultSchema) => getRoutineCreateScript(conn, routine, type, schema),
      truncateAllTables: (_, schema = defaultSchema) => truncateAllTables(conn, schema)
    };
  });
  return _ref.apply(this, arguments);
}

function disconnect(conn) {
  conn.pool.end();
}

function listTables(_x3, _x4) {
  return _listTables.apply(this, arguments);
}

function _listTables() {
  _listTables = _asyncToGenerator(function* (conn, filter) {
    const schemaFilter = (0, _utils.buildSchemaFilter)(filter, 'table_schema');
    const sql = `
    SELECT
      table_schema as schema,
      table_name as name
    FROM information_schema.tables
    WHERE table_type NOT LIKE '%VIEW%'
    ${schemaFilter ? `AND ${schemaFilter}` : ''}
    ORDER BY table_schema, table_name
  `;
    const data = yield driverExecuteQuery(conn, {
      query: sql
    });
    return data.rows;
  });
  return _listTables.apply(this, arguments);
}

function listViews(_x5, _x6) {
  return _listViews.apply(this, arguments);
}

function _listViews() {
  _listViews = _asyncToGenerator(function* (conn, filter) {
    const schemaFilter = (0, _utils.buildSchemaFilter)(filter, 'table_schema');
    const sql = `
    SELECT
      table_schema as schema,
      table_name as name
    FROM information_schema.views
    ${schemaFilter ? `WHERE ${schemaFilter}` : ''}
    ORDER BY table_schema, table_name
  `;
    const data = yield driverExecuteQuery(conn, {
      query: sql
    });
    return data.rows;
  });
  return _listViews.apply(this, arguments);
}

function listRoutines(_x7, _x8) {
  return _listRoutines.apply(this, arguments);
}

function _listRoutines() {
  _listRoutines = _asyncToGenerator(function* (conn, filter) {
    const schemaFilter = (0, _utils.buildSchemaFilter)(filter, 'routine_schema');
    const sql = `
    SELECT
      routine_schema,
      routine_name,
      routine_type
    FROM information_schema.routines
    ${schemaFilter ? `WHERE ${schemaFilter}` : ''}
    GROUP BY routine_schema, routine_name, routine_type
    ORDER BY routine_schema, routine_name
  `;
    const data = yield driverExecuteQuery(conn, {
      query: sql
    });
    return data.rows.map(row => ({
      schema: row.routine_schema,
      routineName: row.routine_name,
      routineType: row.routine_type
    }));
  });
  return _listRoutines.apply(this, arguments);
}

function listTableColumns(_x9, _x10, _x11, _x12) {
  return _listTableColumns.apply(this, arguments);
}

function _listTableColumns() {
  _listTableColumns = _asyncToGenerator(function* (conn, database, table, schema) {
    const sql = `
    SELECT column_name, data_type
    FROM information_schema.columns
    WHERE table_schema = $1
    AND table_name = $2
    ORDER BY ordinal_position
  `;
    const params = [schema, table];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => ({
      columnName: row.column_name,
      dataType: row.data_type
    }));
  });
  return _listTableColumns.apply(this, arguments);
}

function listTableTriggers(_x13, _x14, _x15) {
  return _listTableTriggers.apply(this, arguments);
}

function _listTableTriggers() {
  _listTableTriggers = _asyncToGenerator(function* (conn, table, schema) {
    const sql = `
    SELECT trigger_name
    FROM information_schema.triggers
    WHERE event_object_schema = $1
    AND event_object_table = $2
  `;
    const params = [schema, table];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => row.trigger_name);
  });
  return _listTableTriggers.apply(this, arguments);
}

function listTableIndexes(_x16, _x17, _x18) {
  return _listTableIndexes.apply(this, arguments);
}

function _listTableIndexes() {
  _listTableIndexes = _asyncToGenerator(function* (conn, table, schema) {
    const sql = `
    SELECT indexname as index_name
    FROM pg_indexes
    WHERE schemaname = $1
    AND tablename = $2
  `;
    const params = [schema, table];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => row.index_name);
  });
  return _listTableIndexes.apply(this, arguments);
}

function listSchemas(_x19, _x20) {
  return _listSchemas.apply(this, arguments);
}

function _listSchemas() {
  _listSchemas = _asyncToGenerator(function* (conn, filter) {
    const schemaFilter = (0, _utils.buildSchemaFilter)(filter);
    const sql = `
    SELECT schema_name
    FROM information_schema.schemata
    ${schemaFilter ? `WHERE ${schemaFilter}` : ''}
    ORDER BY schema_name
  `;
    const data = yield driverExecuteQuery(conn, {
      query: sql
    });
    return data.rows.map(row => row.schema_name);
  });
  return _listSchemas.apply(this, arguments);
}

function getTableReferences(_x21, _x22, _x23) {
  return _getTableReferences.apply(this, arguments);
}

function _getTableReferences() {
  _getTableReferences = _asyncToGenerator(function* (conn, table, schema) {
    const sql = `
    SELECT ctu.table_name AS referenced_table_name
    FROM information_schema.table_constraints AS tc
    JOIN information_schema.constraint_table_usage AS ctu
    ON ctu.constraint_name = tc.constraint_name
    WHERE tc.constraint_type = 'FOREIGN KEY' AND tc.table_name = $1
    AND tc.table_schema = $2
  `;
    const params = [table, schema];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => row.referenced_table_name);
  });
  return _getTableReferences.apply(this, arguments);
}

function getTableKeys(_x24, _x25, _x26, _x27) {
  return _getTableKeys.apply(this, arguments);
}

function _getTableKeys() {
  _getTableKeys = _asyncToGenerator(function* (conn, database, table, schema) {
    const sql = `
    SELECT
      tc.constraint_name,
      kcu.column_name,
      CASE WHEN tc.constraint_type LIKE '%FOREIGN%' THEN ctu.table_name
      ELSE NULL
      END AS referenced_table_name,
      tc.constraint_type
    FROM information_schema.table_constraints AS tc
    JOIN information_schema.key_column_usage AS kcu
      USING (constraint_schema, constraint_name)
    JOIN information_schema.constraint_table_usage as ctu
      USING (constraint_schema, constraint_name)
    WHERE tc.table_name = $1
    AND tc.table_schema = $2
    AND tc.constraint_type IN ('PRIMARY KEY', 'FOREIGN KEY')

  `;
    const params = [table, schema];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => ({
      constraintName: row.constraint_name,
      columnName: row.column_name,
      referencedTable: row.referenced_table_name,
      keyType: row.constraint_type
    }));
  });
  return _getTableKeys.apply(this, arguments);
}

function query(conn, queryText) {
  let pid = null;
  let canceling = false;
  const cancelable = (0, _utils2.createCancelablePromise)(_objectSpread(_objectSpread({}, _errors.default.CANCELED_BY_USER), {}, {
    sqlectronError: 'CANCELED_BY_USER'
  }));
  return {
    execute() {
      return runWithConnection(conn, /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (connection) {
          const connClient = {
            connection
          };
          const dataPid = yield driverExecuteQuery(connClient, {
            query: 'SELECT pg_backend_pid() AS pid'
          });
          pid = dataPid.rows[0].pid;

          try {
            const data = yield Promise.race([cancelable.wait(), executeQuery(connClient, queryText)]);
            pid = null;
            return data;
          } catch (err) {
            if (canceling && err.code === pgErrors.CANCELED) {
              canceling = false;
              err.sqlectronError = 'CANCELED_BY_USER';
            }

            throw err;
          } finally {
            cancelable.discard();
          }
        });

        return function (_x28) {
          return _ref2.apply(this, arguments);
        };
      }());
    },

    cancel() {
      return _asyncToGenerator(function* () {
        if (!pid) {
          throw new Error('Query not ready to be canceled');
        }

        canceling = true;

        try {
          const data = yield driverExecuteQuery(conn, {
            query: `SELECT pg_cancel_backend(${pid});`
          });

          if (!data.rows[0].pg_cancel_backend) {
            throw new Error(`Failed canceling query with pid ${pid}.`);
          }

          cancelable.cancel();
        } catch (err) {
          canceling = false;
          throw err;
        }
      })();
    }

  };
}

function executeQuery(_x29, _x30) {
  return _executeQuery.apply(this, arguments);
}

function _executeQuery() {
  _executeQuery = _asyncToGenerator(function* (conn, queryText) {
    const commands = identifyCommands(queryText).map(item => item.type);
    let data = yield driverExecuteQuery(conn, {
      query: queryText,
      multiple: true
    });

    if (!Array.isArray(data)) {
      data = [data];
    }

    return data.filter(result => result.command !== null).map((result, idx) => parseRowQueryResult(result, commands[idx]));
  });
  return _executeQuery.apply(this, arguments);
}

function listDatabases(_x31, _x32) {
  return _listDatabases.apply(this, arguments);
}

function _listDatabases() {
  _listDatabases = _asyncToGenerator(function* (conn, filter) {
    const databaseFilter = (0, _utils.buildDatabseFilter)(filter, 'datname');
    const sql = `
    SELECT datname
    FROM pg_database
    WHERE datistemplate = $1
    ${databaseFilter ? `AND ${databaseFilter}` : ''}
    ORDER BY datname
  `;
    const params = [false];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => row.datname);
  });
  return _listDatabases.apply(this, arguments);
}

function getQuerySelectTop(conn, table, limit, schema) {
  return `SELECT * FROM ${wrapIdentifier(schema)}.${wrapIdentifier(table)} LIMIT ${limit}`;
}

function getTableCreateScript(_x33, _x34, _x35) {
  return _getTableCreateScript.apply(this, arguments);
}

function _getTableCreateScript() {
  _getTableCreateScript = _asyncToGenerator(function* (conn, table, schema) {
    // Reference http://stackoverflow.com/a/32885178
    const params = [table, schema];
    const tableSql = `
    SELECT
      'CREATE TABLE ' || quote_ident(n.nspname) || '.' || quote_ident(c.relname) || ' (' AS create_table
    FROM pg_class c,
    pg_namespace n
    WHERE c.relname = $1
    AND n.nspname = $2
  `;
    let createTable = (yield driverExecuteQuery(conn, {
      query: tableSql,
      params
    })).rows[0].create_table;
    const columnSql = `
    SELECT
      quote_ident(a.attname) AS column_name,
      pg_catalog.format_type(a.atttypid, a.atttypmod) AS type,
      CASE
        WHEN a.attnotnull THEN 'NOT NULL'
      ELSE 'NULL'
      END AS not_null,
      a.attnum as column_order
    FROM pg_class c,
    pg_attribute a,
    pg_type t,
    pg_namespace n
    WHERE c.relname = $1
    AND a.attnum > 0
    AND a.attrelid = c.oid
    AND a.atttypid = t.oid
    AND n.oid = c.relnamespace
    AND n.nspname = $2
    ORDER BY a.attnum ASC
  `;
    const columnData = yield driverExecuteQuery(conn, {
      query: columnSql,
      params
    });
    const columns = [];
    columnData.rows.forEach(row => {
      columns.push(`  ${row.column_name} ${row.type} ${row.not_null}`);
    });
    createTable += `\n${columns.join(',\n')}\n);\n`;
    const constraintSql = `
    SELECT
      CASE WHEN tc.constraint_name IS NULL THEN ''
              ELSE 'ALTER TABLE ' || quote_ident($2) || '.' || quote_ident($1) ||
              ' ADD CONSTRAINT ' || quote_ident(tc.constraint_name)  ||
              ' PRIMARY KEY ' || '(' || constr.column_name || ')'
          END AS constraint
    FROM information_schema.table_constraints tc
    LEFT JOIN information_schema.key_column_usage as constr
    ON constr.constraint_name = tc.constraint_name
    AND constr.table_name = tc.table_name
    AND constr.table_schema = tc.table_schema
    WHERE
      tc.constraint_type  = 'PRIMARY KEY'
      AND tc.table_name   = $1
      AND tc.table_schema = $2;
  `;
    const constraintResult = (yield driverExecuteQuery(conn, {
      query: constraintSql,
      params
    })).rows[0];

    if (constraintResult.constraint.length > 0) {
      createTable += `\n${constraintResult.constraint}`;
    }

    return [createTable];
  });
  return _getTableCreateScript.apply(this, arguments);
}

function getViewCreateScript(_x36, _x37, _x38) {
  return _getViewCreateScript.apply(this, arguments);
}

function _getViewCreateScript() {
  _getViewCreateScript = _asyncToGenerator(function* (conn, view, schema) {
    const createViewSql = `CREATE OR REPLACE VIEW ${wrapIdentifier(schema)}.${view} AS`;
    const sql = 'SELECT pg_get_viewdef($1::regclass, true)';
    const params = [view];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(row => `${createViewSql}\n${row.pg_get_viewdef}`);
  });
  return _getViewCreateScript.apply(this, arguments);
}

function getRoutineCreateScript(_x39, _x40, _x41, _x42) {
  return _getRoutineCreateScript.apply(this, arguments);
}

function _getRoutineCreateScript() {
  _getRoutineCreateScript = _asyncToGenerator(function* (conn, routine, _, schema) {
    let mapFunction;
    let sql;

    if ((0, _utils2.versionCompare)(conn.versionData.version, '8.4') >= 0) {
      sql = `
      SELECT pg_get_functiondef(p.oid)
      FROM pg_proc p
      LEFT JOIN pg_catalog.pg_namespace n ON n.oid = p.pronamespace
      WHERE proname = $1
      AND n.nspname = $2
    `;

      mapFunction = row => row.pg_get_functiondef;
    } else {
      // -- pg_catalog.array_to_string(p.proacl, '\n') AS "Access privileges",
      sql = `
      SELECT
        p.proname,
        n.nspname,
        CASE
          WHEN p.proretset THEN 'SETOF '
          ELSE ''
        END || pg_catalog.format_type(p.prorettype, NULL) as prorettype,
        p.proargnames,
        pg_catalog.oidvectortypes(p.proargtypes) as "proargtypes",
        CASE
          WHEN p.proisagg THEN 'agg'
          WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype THEN 'trigger'
          ELSE 'func'
        END AS protype,
        CASE
          WHEN p.provolatile = 'i' THEN 'IMMUTABLE'
          WHEN p.provolatile = 's' THEN 'STABLE'
          WHEN p.provolatile = 'v' THEN 'VOLATILE'
        END as provolatility,
        pg_catalog.pg_get_userbyid(p.proowner) as prowner_name,
        CASE WHEN prosecdef THEN 'definer' ELSE 'invoker' END AS prosecurity,
        l.lanname,
        p.prosrc,
        pg_catalog.obj_description(p.oid, 'pg_proc') as "description"
      FROM pg_catalog.pg_proc p
          LEFT JOIN pg_catalog.pg_namespace n ON n.oid = p.pronamespace
          LEFT JOIN pg_catalog.pg_language l ON l.oid = p.prolang
      WHERE p.proname = $1
        AND n.nspname = $2
        AND pg_catalog.pg_function_is_visible(p.oid);
    `;

      mapFunction = row => {
        // TODO: expand support for other types as necessary
        if (row.protype !== 'func') {
          return row.prosrc;
        }

        let args = '';

        if (row.proargtypes && row.proargtypes.length > 0) {
          args = (row.proargtypes || '').split(', ').map((val, idx) => `${row.proargnames[idx]} ${val}`).join(', ');
        }

        return `CREATE OR REPLACE FUNCTION ${row.nspname}.${row.proname}(${args})\n  RETURNS ${row.prorettype} AS $$${row.prosrc}$$ LANGUAGE ${row.lanname} ${row.provolatility}`;
      };
    }

    const params = [routine, schema];
    const data = yield driverExecuteQuery(conn, {
      query: sql,
      params
    });
    return data.rows.map(mapFunction);
  });
  return _getRoutineCreateScript.apply(this, arguments);
}

function wrapIdentifier(value) {
  if (value === '*') return value;
  const matched = value.match(/(.*?)(\[[0-9]\])/); // eslint-disable-line no-useless-escape

  if (matched) return wrapIdentifier(matched[1]) + matched[2];
  return `"${value.replace(/"/g, '""')}"`;
}

function getSchema(_x43) {
  return _getSchema.apply(this, arguments);
}

function _getSchema() {
  _getSchema = _asyncToGenerator(function* (conn) {
    const sql = 'SELECT current_schema() AS schema';
    const data = yield driverExecuteQuery(conn, {
      query: sql
    });
    return data.rows[0].schema;
  });
  return _getSchema.apply(this, arguments);
}

function truncateAllTables(_x44, _x45) {
  return _truncateAllTables.apply(this, arguments);
}

function _truncateAllTables() {
  _truncateAllTables = _asyncToGenerator(function* (conn, schema) {
    yield runWithConnection(conn, /*#__PURE__*/function () {
      var _ref3 = _asyncToGenerator(function* (connection) {
        const connClient = {
          connection
        };
        const sql = `
      SELECT quote_ident(table_name) as table_name
      FROM information_schema.tables
      WHERE table_schema = $1
      AND table_type NOT LIKE '%VIEW%'
    `;
        const params = [schema];
        const data = yield driverExecuteQuery(connClient, {
          query: sql,
          params
        });

        if ((0, _utils2.versionCompare)(conn.versionData.version, '8.4') >= 0) {
          const truncateAll = data.rows.map(row => `
        TRUNCATE TABLE ${wrapIdentifier(schema)}.${wrapIdentifier(row.table_name)}
        RESTART IDENTITY CASCADE;
      `).join('');
          yield driverExecuteQuery(connClient, {
            query: truncateAll,
            multiple: true
          });
        } else {
          // RESTART IDENTITY CASCADE was added in Postgres 8.4. The cascade handled
          // under the hood the foreign key constraints so without it, we first have
          // to remove all of them, run the truncate, reset the sequences, and then
          // readd the constraints.
          const seqData = yield driverExecuteQuery(connClient, {
            query: "SELECT relname FROM pg_class WHERE relkind = 'S'"
          });
          const disableTriggers = yield driverExecuteQuery(connClient, {
            query: `
          SELECT 'ALTER TABLE '||quote_ident(nspname)||'.'||quote_ident(relname)||' DROP CONSTRAINT '||quote_ident(conname)||';' AS query
          FROM pg_constraint
          INNER JOIN pg_class ON conrelid=pg_class.oid
          INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace
          WHERE contype='f'
          ORDER BY nspname, relname, conname;
        `
          });
          const enableTriggers = yield driverExecuteQuery(connClient, {
            query: `
          SELECT 'ALTER TABLE "'||nspname||'"."'||relname||'" ADD CONSTRAINT "'||conname||'" '|| pg_get_constraintdef(pg_constraint.oid)||';' AS query
          FROM pg_constraint
          INNER JOIN pg_class ON conrelid=pg_class.oid
          INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace
          WHERE contype='f'
          ORDER BY nspname DESC,relname DESC,conname DESC;
        `
          });
          yield driverExecuteQuery(connClient, {
            query: 'BEGIN'
          });
          let truncateAll = '';
          truncateAll += disableTriggers.rows.map(row => `${row.query}`).join('\n');
          truncateAll += data.rows.map(row => `
        TRUNCATE TABLE ${wrapIdentifier(schema)}.${wrapIdentifier(row.table_name)};
      `).join('');
          truncateAll += seqData.rows.map(row => `
        ALTER SEQUENCE ${row.relname} START 1;
      `).join('');
          truncateAll += enableTriggers.rows.map(row => `${row.query}`).join('\n');

          try {
            yield driverExecuteQuery(connClient, {
              query: truncateAll,
              multiple: true
            });
            yield driverExecuteQuery(connClient, {
              query: 'COMMIT'
            });
          } catch (e) {
            yield driverExecuteQuery(connClient, {
              query: 'ROLLBACK'
            });
            throw e;
          }
        }
      });

      return function (_x48) {
        return _ref3.apply(this, arguments);
      };
    }());
  });
  return _truncateAllTables.apply(this, arguments);
}

function configDatabase(server, database) {
  const config = {
    host: server.config.host,
    port: server.config.port,
    user: server.config.user,
    password: server.config.password,
    database: database.database,
    max: 5 // max idle connections per time (30 secs)

  };

  if (server.sshTunnel) {
    config.host = server.config.localHost;
    config.port = server.config.localPort;
  }

  if (server.config.ssl) {
    config.ssl = server.config.ssl;
  }

  return config;
}

function parseRowQueryResult(data, command) {
  const isSelect = data.command === 'SELECT';
  return {
    command: command || data.command,
    rows: data.rows,
    fields: data.fields,
    rowCount: isSelect ? data.rowCount || data.rows.length : undefined,
    affectedRows: !isSelect && !isNaN(data.rowCount) && data.rowCount !== null ? data.rowCount : undefined
  };
}

function identifyCommands(queryText) {
  try {
    return (0, _sqlQueryIdentifier.identify)(queryText);
  } catch (err) {
    return [];
  }
}

function driverExecuteQuery(conn, queryArgs) {
  const runQuery = connection => {
    const args = {
      text: queryArgs.query,
      values: queryArgs.params,
      multiResult: queryArgs.multiple
    }; // node-postgres has support for Promise query
    // but that always returns the "fields" property empty

    return new Promise((resolve, reject) => {
      connection.query(args, (err, data) => {
        if (err) return reject(err);
        resolve(data);
      });
    });
  };

  return conn.connection ? runQuery(conn.connection) : runWithConnection(conn, runQuery);
}

function runWithConnection(_x46, _x47) {
  return _runWithConnection.apply(this, arguments);
}

function _runWithConnection() {
  _runWithConnection = _asyncToGenerator(function* ({
    pool
  }, run) {
    const connection = yield pool.connect();

    try {
      return yield run(connection);
    } finally {
      connection.release();
    }
  });
  return _runWithConnection.apply(this, arguments);
}