"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _net = _interopRequireDefault(require("net"));

var _ssh = require("ssh2");

var _utils = require("../utils");

var _logger = _interopRequireDefault(require("../logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const logger = (0, _logger.default)('db:tunnel');

function _default(serverInfo) {
  return new Promise( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator(function* (resolve, reject) {
      logger().debug('configuring tunnel');
      const config = yield configTunnel(serverInfo);
      const connections = [];
      logger().debug('creating ssh tunnel server');

      const server = _net.default.createServer( /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (conn) {
          conn.on('error', err => server.emit('error', err));
          logger().debug('creating ssh tunnel client');
          const client = new _ssh.Client();
          connections.push(conn);
          client.on('error', err => server.emit('error', err));
          client.on('ready', () => {
            logger().debug('connected ssh tunnel client');
            connections.push(client);
            logger().debug('forwarding ssh tunnel client output');
            client.forwardOut(config.srcHost, config.srcPort, config.dstHost, config.dstPort, (err, sshStream) => {
              if (err) {
                logger().error('error ssh connection %j', err);
                server.close();
                server.emit('error', err);
                return;
              }

              server.emit('success');
              conn.pipe(sshStream).pipe(conn);
            });
          });

          try {
            const localPort = yield (0, _utils.getPort)();
            logger().debug('connecting ssh tunnel client');
            client.connect(_objectSpread(_objectSpread({}, config), {}, {
              localPort
            }));
          } catch (err) {
            server.emit('error', err);
          }
        });

        return function (_x3) {
          return _ref2.apply(this, arguments);
        };
      }());

      server.once('close', () => {
        logger().debug('close ssh tunnel server');
        connections.forEach(conn => conn.end());
      });
      logger().debug('connecting ssh tunnel server');
      server.listen(config.localPort, config.localHost, err => {
        if (err) return reject(err);
        logger().debug('connected ssh tunnel server');
        resolve(server);
      });
    });

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }());
}

function configTunnel(_x4) {
  return _configTunnel.apply(this, arguments);
}

function _configTunnel() {
  _configTunnel = _asyncToGenerator(function* (serverInfo) {
    const config = {
      username: serverInfo.ssh.user,
      port: serverInfo.ssh.port,
      host: serverInfo.ssh.host,
      dstPort: serverInfo.port,
      dstHost: serverInfo.host,
      sshPort: 22,
      srcPort: 0,
      srcHost: 'localhost',
      localHost: 'localhost',
      localPort: yield (0, _utils.getPort)()
    };
    if (serverInfo.ssh.password) config.password = serverInfo.ssh.password;
    if (serverInfo.ssh.passphrase) config.passphrase = serverInfo.ssh.passphrase;

    if (serverInfo.ssh.privateKey) {
      config.privateKey = yield (0, _utils.readFile)(serverInfo.ssh.privateKey);
    }

    return config;
  });
  return _configTunnel.apply(this, arguments);
}