"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAll = getAll;
exports.add = add;
exports.update = update;
exports.addOrUpdate = addOrUpdate;
exports.removeById = removeById;
exports.decryptSecrects = decryptSecrects;

var _uuid = _interopRequireDefault(require("uuid"));

var _server = require("./validators/server");

var config = _interopRequireWildcard(require("./config"));

var crypto = _interopRequireWildcard(require("./crypto"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function getAll() {
  return _getAll.apply(this, arguments);
}

function _getAll() {
  _getAll = _asyncToGenerator(function* () {
    const _yield$config$get = yield config.get(),
          servers = _yield$config$get.servers;

    return servers;
  });
  return _getAll.apply(this, arguments);
}

function add(_x, _x2) {
  return _add.apply(this, arguments);
}

function _add() {
  _add = _asyncToGenerator(function* (server, cryptoSecret) {
    let srv = _objectSpread({}, server);

    yield (0, _server.validate)(srv);
    const data = yield config.get();
    let newId;

    do {
      newId = _uuid.default.v4();
    } while (!(0, _server.validateUniqueId)(data.servers, newId));

    srv = encryptSecrects(srv, cryptoSecret);
    srv.id = newId;
    data.servers.push(srv);
    yield config.save(data);
    return srv;
  });
  return _add.apply(this, arguments);
}

function update(_x3, _x4) {
  return _update.apply(this, arguments);
}

function _update() {
  _update = _asyncToGenerator(function* (server, cryptoSecret) {
    let srv = _objectSpread({}, server);

    yield (0, _server.validate)(srv);
    const data = yield config.get();
    const index = data.servers.findIndex(item => item.id === srv.id);
    srv = encryptSecrects(srv, cryptoSecret, data.servers[index]);
    data.servers = [...data.servers.slice(0, index), srv, ...data.servers.slice(index + 1)];
    yield config.save(data);
    return server;
  });
  return _update.apply(this, arguments);
}

function addOrUpdate(server, cryptoSecret) {
  const hasId = !!(server.id && String(server.id).length); // TODO: Add validation to check if the current id is a valid uuid

  return hasId ? update(server, cryptoSecret) : add(server, cryptoSecret);
}

function removeById(_x5) {
  return _removeById.apply(this, arguments);
} // ensure all secret fields are encrypted


function _removeById() {
  _removeById = _asyncToGenerator(function* (id) {
    const data = yield config.get();
    const index = data.servers.findIndex(srv => srv.id === id);
    data.servers = [...data.servers.slice(0, index), ...data.servers.slice(index + 1)];
    yield config.save(data);
  });
  return _removeById.apply(this, arguments);
}

function encryptSecrects(server, cryptoSecret, oldServer) {
  const updatedServer = _objectSpread({}, server);

  if (server.password) {
    if (oldServer && oldServer.password && typeof oldServer.password === 'string' && oldServer.encrypted) {
      if (server.password === oldServer.password) {
        updatedServer.password = crypto.unsafeDecrypt(oldServer.password, cryptoSecret);
      }
    }

    if (typeof updatedServer.password === 'string') {
      updatedServer.password = crypto.encrypt(updatedServer.password, cryptoSecret);
    }
  }

  if (server.ssh && server.ssh.password) {
    if (oldServer && oldServer.ssh && oldServer.ssh.password && typeof oldServer.ssh.password === 'string' && oldServer.encrypted) {
      if (server.password === oldServer.password) {
        updatedServer.password = crypto.unsafeDecrypt(oldServer.password, cryptoSecret);
      }
    }

    if (typeof updatedServer.ssh.password === 'string') {
      updatedServer.ssh.password = crypto.encrypt(updatedServer.ssh.password, cryptoSecret);
    }
  }

  updatedServer.encrypted = true;
  return updatedServer;
} // decrypt secret fields


function decryptSecrects(server, cryptoSecret) {
  const updatedServer = _objectSpread({}, server);

  if (!server.encrypted) {
    return server;
  }

  if (server.password && typeof server.password === 'string') {
    updatedServer.password = crypto.unsafeDecrypt(server.password, cryptoSecret);
  } else if (server.password) {
    updatedServer.password = crypto.decrypt(server.password, cryptoSecret);
  }

  if (server.ssh && server.ssh.password && typeof server.ssh.password === 'string') {
    updatedServer.ssh.password = crypto.unsafeDecrypt(server.ssh.password, cryptoSecret);
  } else if (server.ssh && server.ssh.password) {
    updatedServer.ssh.password = crypto.decrypt(server.ssh.password, cryptoSecret);
  }

  updatedServer.encrypted = false;
  return updatedServer;
}