"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfigPath = getConfigPath;
exports.fileExists = fileExists;
exports.fileExistsSync = fileExistsSync;
exports.writeFile = writeFile;
exports.writeJSONFile = writeJSONFile;
exports.writeJSONFileSync = writeJSONFileSync;
exports.readFile = readFile;
exports.readJSONFile = readJSONFile;
exports.readJSONFileSync = readJSONFileSync;
exports.createParentDirectory = createParentDirectory;
exports.createParentDirectorySync = createParentDirectorySync;
exports.resolveHomePathToAbsolute = resolveHomePathToAbsolute;
exports.getPort = getPort;
exports.createCancelablePromise = createCancelablePromise;
exports.versionCompare = versionCompare;

var _fs = _interopRequireDefault(require("fs"));

var _os = require("os");

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _portfinder = _interopRequireDefault(require("portfinder"));

var _envPaths = _interopRequireDefault(require("env-paths"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

let configPath = '';

function getConfigPath() {
  if (configPath) {
    return configPath;
  }

  const configName = 'sqlectron.json';

  const oldConfigPath = _path.default.join((0, _os.homedir)(), `.${configName}`);

  if (process.env.SQLECTRON_HOME) {
    configPath = _path.default.join(process.env.SQLECTRON_HOME, configName);
  } else if (fileExistsSync(oldConfigPath)) {
    configPath = oldConfigPath;
  } else {
    const newConfigDir = (0, _envPaths.default)('Sqlectron', {
      suffix: ''
    }).config;
    configPath = _path.default.join(newConfigDir, configName);
  }

  return configPath;
}

function fileExists(filename) {
  return new Promise(resolve => {
    _fs.default.stat(filename, (err, stats) => {
      if (err) return resolve(false);
      resolve(stats.isFile());
    });
  });
}

function fileExistsSync(filename) {
  try {
    return _fs.default.statSync(filename).isFile();
  } catch (e) {
    return false;
  }
}

function writeFile(filename, data) {
  return new Promise((resolve, reject) => {
    _fs.default.writeFile(filename, data, err => {
      if (err) return reject(err);
      resolve();
    });
  });
}

function writeJSONFile(filename, data) {
  return writeFile(filename, JSON.stringify(data, null, 2));
}

function writeJSONFileSync(filename, data) {
  return _fs.default.writeFileSync(filename, JSON.stringify(data, null, 2));
}

function readFile(filename) {
  const filePath = resolveHomePathToAbsolute(filename);
  return new Promise((resolve, reject) => {
    _fs.default.readFile(_path.default.resolve(filePath), {
      encoding: 'utf-8'
    }, (err, data) => {
      if (err) return reject(err);
      resolve(data);
    });
  });
}

function readJSONFile(filename) {
  return readFile(filename).then(data => JSON.parse(data));
}

function readJSONFileSync(filename) {
  const filePath = resolveHomePathToAbsolute(filename);

  const data = _fs.default.readFileSync(_path.default.resolve(filePath), {
    enconding: 'utf-8'
  });

  return JSON.parse(data);
}

function createParentDirectory(filename) {
  return new Promise((resolve, reject) => (0, _mkdirp.default)(_path.default.dirname(filename), err => err ? reject(err) : resolve()));
}

function createParentDirectorySync(filename) {
  _mkdirp.default.sync(_path.default.dirname(filename));
}

function resolveHomePathToAbsolute(filename) {
  if (!/^~\//.test(filename)) {
    return filename;
  }

  return _path.default.join((0, _os.homedir)(), filename.substring(2));
}

function getPort() {
  return new Promise((resolve, reject) => {
    _portfinder.default.getPort({
      host: 'localhost'
    }, (err, port) => {
      if (err) return reject(err);
      resolve(port);
    });
  });
}

function createCancelablePromise(error, timeIdle = 100) {
  let canceled = false;
  let discarded = false;

  const wait = time => new Promise(resolve => setTimeout(resolve, time));

  return {
    wait() {
      return _asyncToGenerator(function* () {
        while (!canceled && !discarded) {
          // eslint-disable-next-line
          yield wait(timeIdle);
        }

        if (canceled) {
          const err = new Error(error.message || 'Promise canceled.');
          Object.getOwnPropertyNames(error).forEach(key => err[key] = error[key]); // eslint-disable-line no-return-assign

          throw err;
        }
      })();
    },

    cancel() {
      canceled = true;
    },

    discard() {
      discarded = true;
    }

  };
}
/**
 * Compares two version strings.
 *
 * For two version strings, this fucntion will return -1 if the first version is smaller
 * than the second version, 0 if they are equal, and 1 if the second version is smaller.
 * However, this function will only compare up-to the smallest part of the version string
 * defined between the two, such '8' and '8.0.2' will be considered equal.
 *
 * @param {string} a
 * @param {string} b
 * @returns {boolean}
 */


function versionCompare(a, b) {
  const fullA = a.split('.').map(val => parseInt(val, 10));
  const fullB = b.split('.').map(val => parseInt(val, 10));

  for (let i = 0; i < Math.min(fullA.length, fullB.length); i++) {
    if (fullA[i] > fullB[i]) {
      return 1;
    } else if (fullA[i] < fullB[i]) {
      return -1;
    }
  }

  return 0;
}