"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validate = validate;
exports.validateUniqueId = validateUniqueId;

var _valida = _interopRequireDefault(require("valida2"));

var _db = require("../db");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function serverAddressValidator(ctx) {
  const _ctx$obj = ctx.obj,
        host = _ctx$obj.host,
        port = _ctx$obj.port,
        socketPath = _ctx$obj.socketPath; // we need either a host, or a socket path, but not both, and not neither.

  if (!host && !socketPath || host && socketPath) {
    return {
      validator: 'serverAddressValidator',
      msg: 'You must use host or socket path'
    };
  }

  if (socketPath) {
    return undefined;
  }

  if (host && !port || !host && port) {
    return {
      validator: 'serverAddressValidator',
      msg: 'Host and port are required fields.'
    };
  }
}

function clientValidator(ctx, options, value) {
  if (typeof value === 'undefined' || value === null) {
    return undefined;
  }

  if (!_db.CLIENTS.some(dbClient => dbClient.key === value)) {
    return {
      validator: 'clientValidator',
      msg: 'Invalid client type'
    };
  }
}

function boolValidator(ctx, options, value) {
  if (typeof value === 'undefined' || value === null) {
    return undefined;
  }

  if (value !== true && value !== false) {
    return {
      validator: 'boolValidator',
      msg: 'Invalid boolean type.'
    };
  }
}

function passwordSanitizer(ctx, options, value) {
  if (value === undefined || value === null) {
    return value;
  }

  if (typeof value === 'string') {
    return _valida.default.sanitizers.trim(ctx, options, value);
  }

  return {
    ivText: _valida.default.sanitizers.trim(ctx, options, value.ivText),
    encryptedText: _valida.default.sanitizers.trim(ctx, options, value.encryptedText)
  };
}

function passwordValidator(ctx, options, value) {
  if (value === undefined || value === null) {
    return;
  }

  if (typeof value === 'string') {
    return _valida.default.validators.len(ctx, options, value);
  }

  return _valida.default.validators.len(ctx, options, value.ivText) || _valida.default.validators.len(ctx, options, value.encryptedText);
}

const SSH_SCHEMA = {
  host: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  port: [{
    sanitizer: _valida.default.Sanitizer.toInt
  }, {
    validator: _valida.default.Validator.len,
    min: 1,
    max: 5
  }],
  user: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.required
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  password: [{
    sanitizer: passwordSanitizer
  }, {
    validator: passwordValidator,
    min: 1
  }],
  privateKey: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  privateKeyWithPassphrase: [{
    validator: boolValidator
  }]
};
const SERVER_SCHEMA = {
  name: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.required
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  client: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.required
  }, {
    validator: clientValidator
  }],
  ssl: [{
    validator: _valida.default.Validator.required
  }],
  host: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }, {
    validator: serverAddressValidator
  }],
  port: [{
    sanitizer: _valida.default.Sanitizer.toInt
  }, {
    validator: _valida.default.Validator.len,
    min: 1,
    max: 5
  }, {
    validator: serverAddressValidator
  }],
  socketPath: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }, {
    validator: serverAddressValidator
  }],
  database: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  user: [{
    sanitizer: _valida.default.Sanitizer.trim
  }, {
    validator: _valida.default.Validator.len,
    min: 1
  }],
  password: [{
    sanitizer: passwordSanitizer
  }, {
    validator: passwordValidator,
    min: 1
  }],
  ssh: [{
    validator: _valida.default.Validator.schema,
    schema: SSH_SCHEMA
  }]
};
/**
 * validations applied on creating/updating a server
 */

function validate(_x) {
  return _validate.apply(this, arguments);
}

function _validate() {
  _validate = _asyncToGenerator(function* (server) {
    const serverSchema = _objectSpread({}, SERVER_SCHEMA);

    const clientConfig = _db.CLIENTS.find(dbClient => dbClient.key === server.client);

    if (clientConfig && clientConfig.disabledFeatures) {
      clientConfig.disabledFeatures.forEach(item => {
        const _item$split = item.split(':'),
              _item$split2 = _slicedToArray(_item$split, 2),
              region = _item$split2[0],
              field = _item$split2[1];

        if (region === 'server') {
          delete serverSchema[field];
        }
      });
    }

    const validated = yield _valida.default.process(server, serverSchema);

    if (!validated.isValid()) {
      throw validated.invalidError();
    }
  });
  return _validate.apply(this, arguments);
}

function validateUniqueId(servers, serverId) {
  if (!serverId) {
    throw new Error('serverId should be set');
  }

  return !servers.find(srv => srv.id === serverId);
}